"""A {{ class }} Command."""
from cleo import Command


class {{ class }}(Command):
    """
    Description of command

    command:name
        {argument : description}
    """

    def handle(self):
        pass

