"""A {{ class }} Queue Job."""

from masonite.queues import Queueable


class {{ class }}(Queueable):
    """A {{ class }} Job."""

    def __init__(self):
        """A {{ class }} Constructor."""
        pass

    def handle(self):
        """Logic to handle the job."""
        pass

