{% extends 'auth/base.html' %}

{% block content %}
<div class="col-xs-4 col-xs-offset-4">
    <div class="panel panel-default">
        <div class="panel-heading">Verified</div>
        <div class="panel-body">
            Thank you for confirming your email. Click here to go to the <a href="/home">home</a> page.
        </div>
    </div>
</div>
{% endblock %}
