{% extends 'auth/base.html' %}

{% block content %}
<div class="row">
    <div class="cover-container d-flex h-100 p-3 mx-auto my-5 flex-column">
        <h1 class="cover-heading">Dashboard</h1>
        <p class="lead">This is a dashboard. Hello <strong>{{ auth().name }}</strong></p>
    </div>
</div>
{% endblock %}
