{% extends 'auth/base.html' %}

{% block content %}
<div class="col-xs-4 col-xs-offset-4">
    <div class="panel panel-default">
        <div class="panel-heading">Verify</div>
        <div class="panel-body">
            Please check your email and follow the link to verify your email. If
            you need us to resend the email. Click <a href="/email/verify/send">here</a>
        </div>
    </div>
</div>
{% endblock %}
