{% extends 'auth/base.html' %}

{% block content %}
<div class="col-xs-4 col-xs-offset-4">
    <div class="panel panel-default">
        <div class="panel-heading">Verifying Error</div>
        <div class="panel-body">
            Confirming email failed. Click here to go <a href="/home">home</a>
        </div>
    </div>
</div>
{% endblock %}
